function love.load()
	movetick = 0
	geneticAttributes = {1,2,3,7,9,14}
	math.randomseed(os.time())
	mobs = {}
	loadMobs(2)
	food = {}
	loadFoodSources(7)
	love.graphics.setBackgroundColor(0,200,20)
end

function love.update(dt)
	--iterate over mobs
	lowerHunger()
	checkEating()
	addFood()
	growFood()
	if love.timer.getTime() - movetick > 0.15 then
		for x=1,#mobs do
			--if it's eating, and not up to it's turning point + extra then keep eating
			--if it's below it's turning point, then go eat.
			if mobs[x][6] <= mobs[x][9] and mobs[x][5] == 0 then -- if they're hungry
				if mobs[x][15] == 1 or mobs[x][15] == 3 then --if they aren't in route to a place
					findClosestNode(x)
				end
			elseif mobs[x][6] < mobs[x][9] + mobs[x][14] and mobs[x][10] then --if they're still eating
					findClosestNode(x)
			elseif mobs[x][6] > mobs[x][9] and mobs[x][5] == 0 then --if they're hungry and need a new direction
				newDirection(x)
			end
			if mobs[x][4] == 1 then--moving right
				right(x)
			elseif mobs[x][4] == 2 then--moving down
				down(x)
			elseif mobs[x][4] == 3 then --moving left
				left(x)
			elseif mobs[x][4] == 4 then --moving up
				up(x)
			elseif mobs[x][4] == 0 then
				--stay and eat
				mobs[x][5] = 0
			end
		end
		movetick = love.timer.getTime()
	end
	--determine the angle in radians at which they move
		--for now, i'll make it so they only move at 90 degree angles
end

function love.draw()
	for x = 1, #food do
		love.graphics.setColor(0,100,0)
		love.graphics.circle("fill", food[x][1], food[x][2], food[x][3] * 3)
		love.graphics.setColor(0,0,0)
		love.graphics.print(food[x][3], food[x][1] - 5, food[x][2] + 5)
		love.graphics.print(x, food[x][1] - 5, food[x][2] - 5)
	end
	love.graphics.setColor(0,0,0)
	love.graphics.print(love.timer.getTime(),10,10)
	love.graphics.print(#mobs,10,20)
	love.graphics.print(#food,10,30)
	love.graphics.setColor(0,0,0)
	for x = 1,#mobs do
		love.graphics.circle("line", mobs[x][1], mobs[x][2], 35)
		love.graphics.print(mobs[x][6] .. ":" .. mobs[x][9], mobs[x][1] - 5, mobs[x][2] - 5)
		love.graphics.print(mobs[x][11], mobs[x][1] - 5, mobs[x][2] + 5)
	end
end

function love.mousepressed(x,y,button)
	local xd = tonumber(x)
	local yd = tonumber(y)
	if button == "l" then
		newMob(xd,yd)
	else
		newFoodSource(xd,yd)
	end
end

--[[-----------------
----New Functions----
-----------------]]--

function loadFoodSources(n)
	for x = 1, n do
		newFoodSource()
	end
end

function newFoodSource(x,y)
	if type(x) ~= "number" then
		x = math.random(1,love.graphics.getWidth())
	end
	if type(y) ~= "number" then
		y = math.random(1,love.graphics.getHeight())
	end
	food[(#food + 1)] = {
	x, -- 1
	y, -- 2
	math.random(5,10), --intensity 3
	math.random(3,6), --time to grow back after being reduced to 0 intensity 4
	0, --growback timer 5
	math.random(8,13), --max intensity 6
	math.random(2,7) --number of times it will regrow from zero
	}
	stabilizeFoodIntensity()
end

function stabilizeFoodIntensity()
	for x = 1, #food do
		if food[x][3] > food[x][6] then
			food[x][3] = food[x][6]
		end
	end
end

function growFood()
	for x = 1, #food do
		if love.timer.getTime() - food[x][5] > food[x][4] then
			--i guess they don't have a max intensity
			if food[x][3] < food[x][6] then
				food[x][3] = food[x][3] + 1
				food[x][5] = love.timer.getTime()
			end
		end
	end
end

function loadMobs(n)
	for x = 1, n do
		newMob()
	end
end

function newMob(x,y)
	if type(x) ~= "number" then
		x = math.random(1,love.graphics.getWidth())
	end
	if type(y) ~= "number" then
		y = math.random(1,love.graphics.getHeight())
	end
	local z = love.timer.getTime()
	mobs[( #mobs + 1 )] = {
	x, --x pos 1
	y, --y pos 2
	math.random(5,9), --speed 3
	math.random(1,4),			   --angle 4
	math.random(40,100),  --distance 5
	15, --hunger 6
	math.random(2,6), --foodtick 7
	0, --foodtimer 8 WTF MAN
	math.random(4,7), --hunger boundary 9
	false, --eating 10
	0, --node it's currently eating from 11
	0, --time at last consumption 12
	math.random(130,180), --eyesight 13
	math.random(3,8), --how far it will eat past it's hunger boundary 14
	3 --whether it's going towards a node (no,yes,random) 15
	}
end

function lowerHunger()
	for x = 1, #mobs do
		if love.timer.getTime() - mobs[x][8] > mobs[x][7] then
			mobs[x][6] = mobs[x][6] - 1
			mobs[x][8] = love.timer.getTime()
		end
	end
end

function findClosestNode(mob)
	--[[
	look around within the mob's eyesight
	if there's a node within their eyesight, go to it
	later, will add functionality to not go to a node occupied by another mob
		or at least the least number of mobs
	also, will add functionality to go to a node with a higher intensity even
		if it is farther away
	if they are on the verge of starving, they will go to the closest node regardless
		of competition or node intensity. maybe not regardless of competition
	--]]
	
	local mobx = mobs[mob][1] -- for convenience
	local moby = mobs[mob][2]
	local sight = mobs[mob][13]
	local distance = 10000 --distance to the closest node
	local d = 0
	local node = -1 -- closest node
	local dir = 0 --direction to take
	for x = 1, #food do
		if food[x][1] < mobx  and food[x][2] < moby then --to the upper left of the mob
			if mobx - food[x][1] < sight and moby - food[x][2] < sight then
				--calc distance
					local fooddistance = (mobx - food[x][1]) + (moby - food[x][2])
				--compare it to the shortest distance found
					if fooddistance < distance then
						distance = fooddistance
						node = x
						if (mobx - food[x][1]) < (moby - food[x][2]) then
							dir = 4 -- y
							d = moby - food[x][2]
						else
							dir = 3 --x
							d = mobx - food[x][1]
						end
					end
			end
		elseif food[x][1] > mobx  and food[x][2] < moby then --to the upper right of the mob
			if food[x][1] - mobx < sight and moby - food[x][2] < sight then
				--calc distance
					local fooddistance = (food[x][1] - mobx) + (moby - food[x][2])
				--compare it to the shortest distance found
					if fooddistance < distance then
						distance = fooddistance
						node = x
						if (food[x][1] - mobx) < (moby - food[x][2]) then
							dir = 4 --y
							d = moby - food[x][2]
						else
							dir = 1 --x
							d = food[x][1] - mobx
						end
					end
			end
		elseif food[x][1] < mobx  and food[x][2] > moby then --to the lower left of the mob
			if mobx - food[x][1] < sight and food[x][2] - moby < sight then
				--calc distance
					local fooddistance = (mobx - food[x][1]) + (food[x][2] - moby)
				--compare it to the shortest distance found
					if fooddistance < distance then
						distance = fooddistance
						node = x
						if (mobx - food[x][1]) < (food[x][2] - moby) then
							dir = 2 --y
							d = food[x][2] - moby
						else
							dir = 1 --x
							d = mobx - food[x][1]
						end
					end
			end
		elseif food[x][1] > mobx  and food[x][2] > moby then --to the lower right of the mob
			if food[x][1] - mobx < sight and food[x][2] - moby < sight then
				--calc distance
					local fooddistance = (food[x][1] - mobx) + (food[x][2] - moby)
				--compare it to the shortest distance found
					if fooddistance < distance then
						distance = fooddistance
						node = x
						if (food[x][1] - mobx) < (food[x][2] - moby) then
							dir = 2 --y
							d = food[x][2] - moby
						else
							dir = 1 --x
							d = food[x][1] - mobx
						end
					end
			end
		end
	end
	--outside the for loop
	--find which direction (x or y) needs to be travelled the farthest
	-- 4 = angle 5 = distance
	if node ~= -1 then
		mobs[mob][4] = dir
		mobs[mob][5] = d
		mobs[mob][15] = 2
	else
		--go in a random direction until it can see food
		--or possibly follow another mob
		newDirection(mob)
		mobs[mob][15] = 3
	end
end

function addFood()
	for x = 1, #mobs do
		if mobs[x][10] == true then --if eating
			if mobs[x][6] < 15 then --if hunger is less then 15
				if love.timer.getTime() - mobs[x][12] > 0.35 then --it can eat every 0.35 seconds
					if food[mobs[x][11]][3] > 0 then --if the node has food to give
						mobs[x][6] = mobs[x][6] + 1 --give the mob food
						food[mobs[x][11]][3] = food[mobs[x][11]][3] - 1 --take food from the node
						mobs[x][12] = love.timer.getTime() --reset eat time of the mob
					end
				end
			end
		end
	end
end

function newDirection(mob)
	mobs[mob][4] = math.random(1,4) --angle
	mobs[mob][5] = math.random(40,100) --distance
end

function checkEating()
	for x = 1, #mobs do
		eating(x)
	end
end

function eating(mob)
	local mobx = mobs[mob][1] -- for convenience
	local moby = mobs[mob][2]
	local eat = false
	local node = 0
	for x = 1, #food do
		if food[x][1] < mobx  and food[x][2] < moby then --to the upper left of the mob
			if mobx - food[x][1] < 35 and moby - food[x][2] < 35 then
				eat = true
				node = x
			end
		elseif food[x][1] > mobx  and food[x][2] < moby then --to the upper right of the mob
			if food[x][1] - mobx < 35 and moby - food[x][2] < 35 then
				eat = true
				node = x
			end
		elseif food[x][1] < mobx  and food[x][2] > moby then --to the lower left of the mob
			if mobx - food[x][1] < 35 and food[x][2] - moby < 35 then
				eat = true
				node = x
			end
		elseif food[x][1] > mobx  and food[x][2] > moby then --to the lower right of the mob
			if food[x][1] - mobx < 35 and food[x][2] - moby < 35 then
				eat = true
				node = x
			end
		end
		
	end
	mobs[mob][10] = eat
	mobs[mob][11] = node
end

function right(mob)
	if mobs[mob][3] > love.graphics.getWidth() - mobs[mob][1] then --wall
		mobs[mob][5] = 0
		mobs[mob][1] = love.graphics.getWidth()
	else
		if mobs[mob][5] > mobs[mob][3] then --if distance is greater than speed
			mobs[mob][1] = mobs[mob][1] + mobs[mob][3] --update x coord
			mobs[mob][5] = mobs[mob][5] - mobs[mob][3] --update distance remaining
		else
			mobs[mob][1] = mobs[mob][1] + mobs[mob][5]
			mobs[mob][5] = 0
		end
	end
end

function down(mob)
	if mobs[mob][3] > love.graphics.getHeight() - mobs[mob][2] then
		mobs[mob][5] = 0
		mobs[mob][2] = love.graphics.getHeight()
	else
		if mobs[mob][5] > mobs[mob][3] then --if distance is greater than speed
			mobs[mob][2] = mobs[mob][2] + mobs[mob][3] --update y coord
			mobs[mob][5] = mobs[mob][5] - mobs[mob][3] --update distance remaining
		else
			mobs[mob][2] = mobs[mob][2] + mobs[mob][5]
			mobs[mob][5] = 0
		end
	end
end

function left(x)
	if mobs[x][3] > mobs[x][1] then
		mobs[x][5] = 0
		mobs[x][1] = 0
	else
		if mobs[x][5] > mobs[x][3] then --if distance is greater than speed
			mobs[x][1] = mobs[x][1] - mobs[x][3] --update x coord
			mobs[x][5] = mobs[x][5] - mobs[x][3] --update distance remaining
		else
			mobs[x][1] = mobs[x][1] - mobs[x][5]
			mobs[x][5] = 0
		end
	end
end

function up(x)
	if mobs[x][3] > mobs[x][2] then
		mobs[x][5] = 0
		mobs[x][2] = 0
	else
		if mobs[x][5] > mobs[x][3] then --if distance is greater than speed
			mobs[x][2] = mobs[x][2] - mobs[x][3] --update y coord
			mobs[x][5] = mobs[x][5] - mobs[x][3] --update distance remaining
		else
			mobs[x][2] = mobs[x][2] - mobs[x][5]
			mobs[x][5] = 0
		end
	end
end